@@ -22,9 +22,10 @@
 #include "config.h"
 #endif
 
-#include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4util/libxfce4util.h>
 
+#include "libxfce4ui/libxfce4ui.h"
+
 #include "xfce-shortcuts-xfwm4.h"
 #include "xfce-shortcuts.h"
 
