@@ -16,22 +16,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-
-#include <gtk/gtk.h>
-
-#if G_GNUC_CHECK_VERSION (4, 6)
+#if G_GNUC_CHECK_VERSION(4, 6)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wundef"
 #endif
 #include <glibtop/mem.h>
-#if G_GNUC_CHECK_VERSION (4, 6)
+#if G_GNUC_CHECK_VERSION(4, 6)
 #pragma GCC diagnostic pop
 #endif
 
@@ -46,9 +36,15 @@ enum
   KERNEL
 };
 
-char *get_os_type (void);
-char *get_system_info (guint infotype);
-char *get_cpu_info (const glibtop_sysinfo *info);
-char *get_gpu_info (guint *num_gpus);
-char *get_os_name (void);
-char *get_os_logo (void);
+char *
+get_os_type (void);
+char *
+get_system_info (guint infotype);
+char *
+get_cpu_info (const glibtop_sysinfo *info);
+char *
+get_gpu_info (guint *num_gpus);
+char *
+get_os_name (void);
+char *
+get_os_logo (void);
